@@ -20,16 +20,18 @@ Functions for linking and combining different datasets
 
 from __future__ import absolute_import, print_function
 
-from .core import get_config, _data_out, get_obj_if_Acc
-from .utils import read_csv_if_string, parmap, get_name
-from .duke import duke
-from .cleaning import clean_technology
-
-import os.path
-import pandas as pd
-import numpy as np
-from itertools import combinations
 import logging
+import os.path
+from itertools import combinations
+
+import numpy as np
+import pandas as pd
+
+from .cleaning import clean_technology
+from .core import get_config, get_obj_if_Acc
+from .duke import duke
+from .utils import get_name, parmap, read_csv_if_string
+
 logger = logging.getLogger(__name__)
 
 
@@ -43,14 +45,13 @@ def best_matches(links):
     links : pd.DataFrame
         Links as returned by duke
     """
-    labels = links.columns.difference({'scores'})
-    return (links
-            .groupby(links.iloc[:, 1], as_index=False, sort=False)
-            .apply(lambda x: x.loc[x.scores.idxmax(), labels]))
+    labels = links.columns.difference({"scores"})
+    return links.groupby(links.iloc[:, 1], as_index=False, sort=False).apply(
+        lambda x: x.loc[x.scores.idxmax(), labels]
+    )
 
 
-def compare_two_datasets(dfs, labels, use_saved_matches=False,
-                         country_wise=True, config=None, **dukeargs):
+def compare_two_datasets(dfs, labels, country_wise=True, config=None, **dukeargs):
     """
     Duke-based horizontal match of two databases. Returns the matched
     dataframe including only the matched entries in a multi-indexed
@@ -76,37 +77,40 @@ def compare_two_datasets(dfs, labels, use_saved_matches=False,
     if config is None:
         config = get_config()
 
+    deprecated_args = {"use_saved_matches", "use_saved_aggregation"}
+    used_deprecated_args = deprecated_args.intersection(dukeargs)
+    if used_deprecated_args:
+        for arg in used_deprecated_args:
+            dukeargs.pop(arg)
+        msg = "The following arguments were deprecated and are being ignored: "
+        logger.warn(msg + f"{used_deprecated_args}")
+
     dfs = list(map(read_csv_if_string, dfs))
-    if not ('singlematch' in dukeargs):
-        dukeargs['singlematch'] = True
-    saving_path = _data_out('matches/matches_{}_{}.csv'
-                            .format(*np.sort(labels)), config=config)
-    if use_saved_matches:
-        if os.path.exists(saving_path):
-            logger.info('Reading saved matches for dfs {} and {}'
-                        .format(*labels))
-            return pd.read_csv(saving_path, index_col=0)
-        else:
-            logger.warning("Non-existing saved matches for dataset '{}', '{}'"
-                           " continuing by matching again".format(*labels))
+    if not ("singlematch" in dukeargs):
+        dukeargs["singlematch"] = True
 
     def country_link(dfs, country):
         # country_selector for both dataframes
-        sel_country_b = [df['Country'] == country for df in dfs]
+        sel_country_b = [df["Country"] == country for df in dfs]
         # only append if country appears in both dataframse
         if all(sel.any() for sel in sel_country_b):
-            return duke([df[sel] for df, sel in zip(dfs, sel_country_b)],
-                        labels, **dukeargs)
+            return duke(
+                [df[sel] for df, sel in zip(dfs, sel_country_b)], labels, **dukeargs
+            )
         else:
             return pd.DataFrame()
 
     if country_wise:
-        countries = config['target_countries']
+        countries = config["target_countries"]
         links = pd.concat([country_link(dfs, c) for c in countries])
     else:
         links = duke(dfs, labels=labels, **dukeargs)
-    matches = best_matches(links)
-    matches.to_csv(saving_path)
+
+    if links.empty:
+        matches = pd.DataFrame(columns=labels)
+    else:
+        matches = best_matches(links)
+
     return matches
 
 
@@ -137,23 +141,34 @@ def cross_matches(sets_of_pairs, labels=None):
         match_base = pd.concat(base, axis=1).reset_index()
         matches = pd.concat([matches, match_base], sort=True)
 
+    assert not matches.empty, "No matches found"
+
+    if matches.isnull().all().any():
+        cols = ", ".join(matches.columns[matches.isnull().all()])
+        raise ValueError(f"No matches found for data source {cols}")
+
     matches = matches.drop_duplicates().reset_index(drop=True)
     for i in labels:
-        matches = pd.concat([
-            matches.groupby(i, as_index=False, sort=False)
-                   .apply(lambda x: x.loc[x.isnull().sum(axis=1).idxmin()]),
-            matches[matches[i].isnull()]
-        ]).reset_index(drop=True)
-    return (matches
-            .assign(length=matches.notna().sum(axis=1))
-            .sort_values(by='length', ascending=False)
-            .reset_index(drop=True)
-            .drop('length', axis=1)
-            .reindex(columns=labels))
+        matches = pd.concat(
+            [
+                matches.groupby(i, as_index=False, sort=False).apply(
+                    lambda x: x.loc[x.isnull().sum(axis=1).idxmin()]
+                ),
+                matches[matches[i].isnull()],
+            ]
+        ).reset_index(drop=True)
+    return (
+        matches.assign(length=matches.notna().sum(axis=1))
+        .sort_values(by="length", ascending=False)
+        .reset_index(drop=True)
+        .drop("length", axis=1)
+        .reindex(columns=labels)
+    )
 
 
-def link_multiple_datasets(datasets, labels, use_saved_matches=False,
-                           config=None, **dukeargs):
+def link_multiple_datasets(
+    datasets, labels, use_saved_matches=False, config=None, **dukeargs
+):
     """
     Duke-based horizontal match of multiple databases. Returns the
     matching indices of the datasets. Compares all properties of the
@@ -181,10 +196,8 @@ def link_multiple_datasets(datasets, labels, use_saved_matches=False,
     combs = list(combinations(range(len(labels)), 2))
 
     def comp_dfs(dfs_lbs):
-        logger.info('Comparing {0} with {1}'.format(*dfs_lbs[2:]))
-        return compare_two_datasets(dfs_lbs[:2], dfs_lbs[2:],
-                                    use_saved_matches=use_saved_matches,
-                                    config=config, **dukeargs)
+        logger.info("Comparing data sources `{0}` and `{1}`".format(*dfs_lbs[2:]))
+        return compare_two_datasets(dfs_lbs[:2], dfs_lbs[2:], config=config, **dukeargs)
 
     mapargs = [[dfs[c], dfs[d], labels[c], labels[d]] for c, d in combs]
     all_matches = parmap(comp_dfs, mapargs)
@@ -192,8 +205,7 @@ def link_multiple_datasets(datasets, labels, use_saved_matches=False,
     return cross_matches(all_matches, labels=labels)
 
 
-def combine_multiple_datasets(datasets, labels=None, use_saved_matches=False,
-                              config=None, **dukeargs):
+def combine_multiple_datasets(datasets, labels=None, config=None, **dukeargs):
     """
     Duke-based horizontal match of multiple databases. Returns the
     matched dataframe including only the matched entries in a
@@ -232,19 +244,18 @@ def combine_multiple_datasets(datasets, labels=None, use_saved_matches=False,
         """
         datasets = list(map(read_csv_if_string, datasets))
         for i, data in enumerate(datasets):
-            datasets[i] = (data
-                           .reindex(cross_matches.iloc[:, i])
-                           .reset_index(drop=True))
-        return (pd.concat(datasets, axis=1,
-                          keys=cross_matches.columns.tolist())
-                .reorder_levels([1, 0], axis=1)
-                .reindex(columns=config['target_columns'], level=0)
-                .reset_index(drop=True))
-    crossmatches = link_multiple_datasets(datasets, labels,
-                                          use_saved_matches=use_saved_matches,
-                                          config=config, **dukeargs)
-    return (combined_dataframe(crossmatches, datasets, config)
-            .reindex(columns=config['target_columns'], level=0))
+            datasets[i] = data.reindex(cross_matches.iloc[:, i]).reset_index(drop=True)
+        return (
+            pd.concat(datasets, axis=1, keys=cross_matches.columns.tolist())
+            .reorder_levels([1, 0], axis=1)
+            .reindex(columns=config["target_columns"], level=0)
+            .reset_index(drop=True)
+        )
+
+    crossmatches = link_multiple_datasets(datasets, labels, config=config, **dukeargs)
+    return combined_dataframe(crossmatches, datasets, config).reindex(
+        columns=config["target_columns"], level=0
+    )
 
 
 def reduce_matched_dataframe(df, show_orig_names=False, config=None):
@@ -265,24 +276,34 @@ def reduce_matched_dataframe(df, show_orig_names=False, config=None):
 
     # define which databases are present and get their reliability_score
     sources = df.columns.levels[1]
-    rel_scores = pd.Series({s: config[s]['reliability_score'] for s in sources})\
-                   .sort_values(ascending=False)
-    cols = config['target_columns']
-    props_for_groups = {col: 'first'
-                        for col in cols}
-    props_for_groups.update({'YearCommisisoned': 'min',
-                             'DateRetrofit': 'max',
-                             'projectID': lambda x: dict(x.droplevel(0).dropna()),
-                             'eic_code': 'unique'})
+    rel_scores = pd.Series(
+        {s: config[s]["reliability_score"] for s in sources}, dtype=float
+    ).sort_values(ascending=False)
+    cols = config["target_columns"]
+    props_for_groups = {col: "first" for col in cols}
+    props_for_groups.update(
+        {
+            "DataIn": "min",
+            "DateRetrofit": "max",
+            "DataOut": "max",
+            "projectID": lambda x: dict(x.droplevel(0).dropna()),
+            "eic_code": set,
+        }
+    )
     props_for_groups = pd.Series(props_for_groups)[cols].to_dict()
 
-    # set low priority on Fueltype 'Other'
+    # set low priority on Fueltype 'Other' and Set 'PP'
     # turn it since aggregating only possible for axis=0
-    sdf = df.replace({'Fueltype': {'Other': np.nan}})\
-            .stack(1).reindex(rel_scores.index, level=1)\
-            .groupby(level=0)\
-            .agg(props_for_groups)\
-            .replace({'Fueltype': {np.nan: 'Other'}})
+    sdf = (
+        df.assign(Set=lambda df: df.Set.where(df.Set != "PP"))
+        .assign(Fueltype=lambda df: df.Fueltype.where(df.Set != "Other"))
+        .stack(1)
+        .reindex(rel_scores.index, level=1)
+        .groupby(level=0)
+        .agg(props_for_groups)
+        .assign(Set=lambda df: df.Set.fillna("PP"))
+        .assign(Fueltype=lambda df: df.Fueltype.fillna("Other"))
+    )
 
     if show_orig_names:
         sdf = sdf.assign(**dict(df.Name))
